# DB_EGADS.pm              -*- Perl -*-
#
# The basic EGADS Scheduling Database interface
# module.
#
package DB_EGADS;

require 5.004_04;

use strict;

use Debug qw(debug addToSumFile);
use Data::Dumper;
use DB_File;
use Carp qw(carp cluck croak confess);

my $VERSION = do {my @r=(q$Revision: 1.1 $=~/\d+/g);sprintf "%d."."%02d"x$#r,@r};

sub new {
	my $class = shift;
	my $dbFileName = shift || "/tmp/EGADS_DB";

	my $self = bless {}, $class;

	$self->{dbFileName} = $dbFileName;
	$self->_open();


	$self->cleanupDB();

	$self;
}

sub collapseSpans {
	my $self = shift;
	my $spans = shift;

	return unless(defined($spans));
	my @spans = @{$spans};
	my %tmp;

	debug(100,"DB_EGADS::collapseSpans: Working with $#spans\n");
	foreach my $span (@spans) {
		my $start = $span->{start_time};
		my $stop = $span->{stop_time};
		next if ($stop < $start);
		next if ($tmp{$start} && ($stop == $tmp{$start}->{stop}));
		if ($tmp{$start} && ($stop > $tmp{$start}->{stop})) {
			$tmp{$start}->{stop} = $stop;
			next;
		}
		next if (($start eq "") || ($stop eq ""));
		$tmp{$start} = { stop => $stop };
	}


	my $found = 1;
	my $entrynum = 0;
	debug(75,"DB_EGADS::collapseSpans: while(1) loop entering\n");
	while (1) {

		$entrynum++ unless ($found);
		my @start_times = sort {$a <=> $b} keys(%tmp);
		last if ($#start_times <= $entrynum);

		my $start = $start_times[$entrynum]; # this is the earliest time
		my $stop = $tmp{$start_times[$entrynum]}->{stop}; # this is the unknown

		$found = 0;
		for (my $i = $entrynum+1; $i <= $#start_times; $i++) {
			if (($stop + 1) >= $start_times[$i]) {
				if ($tmp{$start_times[$i]}->{stop} > $stop) {
					$stop = $tmp{$start_times[$i]}->{stop};
				}
				$tmp{$start_times[$entrynum]}->{stop} = $stop;
				delete($tmp{$start_times[$i]});
				$found = 1;
				last;
			}
		}
	}
	my $return = [];
	foreach my $start (keys(%tmp)) {
		push (@{$return} , { start_time => $start, stop_time => $tmp{$start}->{stop} });
	}
	debug(100,"DB_EGADS::collapseSpans: Returning $#{$return}\n");
	return($return);
}

sub findOpenSpans {
	my $self = shift;
	my $site = shift;
	my $taskid = shift;
	my $start = shift;
	my $stop = (@_) ? shift : time();

	my $key = $site . "-" . $taskid;

	debug(100,"DB_EGADS::findOpenSpans: Entering (key=$key)...\n");
	$self->cleanupDBWithKey($key);

	my $chkspans = [];

	push(@{$chkspans},{start_time => $start, stop_time => $stop});

	my @times = $self->{DB}->get_dup($key);
	debug(100,"DB_EGADS::findOpenSpans: $#times spans to check\n");
	foreach my $span (@times) {
		my ($start, $stop) = split(/\|/, $span);
		debug(100,"DB_EGADS::findOpenSpans: found span $start -> $stop\n");
		my @tmpnew;
		foreach my $chk (@{$chkspans}) {
#conditions:
# span outside of range
			next if ($chk->{start_time} == $chk->{stop_time});
			if (($start > $chk->{stop_time}) || ($stop < $chk->{start_time})) {
				push(@tmpnew,$chk);
				next;
			}
# span covers all of range
			if (($start <= $chk->{start_time}) && ($stop >= $chk->{stop_time})) {
				next;
			}
# span covers start of range
			if (($start <= $chk->{start_time}) && ($stop < $chk->{stop_time})) {
				push(@tmpnew,{start_time => $stop+1, stop_time => $chk->{stop_time}});
				next;
			}
# span covers end of range
			if (($start > $chk->{start_time}) && ($stop >= $chk->{stop_time})) {
				push(@tmpnew,{start_time => $chk->{start_time}, stop_time => $start-1});
				next;
			}
			if (($start > $chk->{start_time}) && ($stop < $chk->{stop_time})) {
				push(@tmpnew,{start_time => $chk->{start_time}, stop_time => $start-1});
				push(@tmpnew,{start_time => $stop+1, stop_time => $chk->{stop_time}});
				next;
			}
		}
		$chkspans = [];
		foreach my $tmp (@tmpnew) {
			unless ($tmp->{start_time} == $tmp->{stop_time}) {
				push(@{$chkspans},$tmp);
			}
		}
		
# span splits range
	}
	debug(100,"DB_EGADS::findOpenSpans: Exiting...\n");
	return($chkspans);

}
sub cleanupDBWithKey {
	my $self = shift;
	my $key = shift;

	debug(60,"DB_EGADS::cleanupDBWithKey: Entering (key = $key)...\n");
	my @times = $self->{DB}->get_dup($key);
	if ($#times <= 0) {
		debug(100,"DB_EGADS::cleanupDBWithKey: Spans already clean\n");
		return(0);
	}
	my $tmp = [];
	$self->{DB}->del($key);
	foreach my $span (@times) {
		my ($start, $stop) = split(/\|/, $span);
		push(@{$tmp}, {start_time => $start, stop_time => $stop});
	}

	my $newspans = $self->collapseSpans($tmp);

	debug(100,"DB_EGADS::cleanupDBWithKey: Collapsed from $#{$tmp} to  $#{$newspans} spans\n");
	foreach my $span (@{$newspans}) {
		$self->addSpanWithKey($key,$span->{start_time},$span->{stop_time});
	}
	$self->{DB}->sync();
	debug(60,"DB_EGADS::cleanupDBWithKey: Exiting (key = $key)...\n");
}

sub cleanupDB {
	my $self = shift;

	foreach my $key (keys(%{$self->{_DB}})) {
		$self->cleanupDBWithKey($key);
	}
}

sub DESTROY {
	my $self = shift;

	return $self->_close();

}

sub _close {
	my $self = shift;
	
	if (ref($self->{_DB}) eq "HASH") {
		undef $self->{DB};
		untie $self->{_DB};
	}
	return 1;
}

sub getEndTimeForKey {
	my $self = shift;
	my $key = shift;
	my @times = $self->{DB}->get_dup($key);

	my ($DBstart, $DBstop) = split(/\|/,$times[$#times]);

	return($DBstop);
	
}


sub isOpenSpan {
	my $self = shift;
	my $site = shift;
	my $taskid = shift;
	my $systemtime = shift;
	my $maxback = shift;

	my $key = $site . "-" . $taskid;

	$self->cleanupDBWithKey($key);

# get existing task times
	my @times = $self->{DB}->get_dup($key);

# if no times exist, there is an open time
	return(1) if ($#times < 0 );

# should only need to check last span since cleanup will consolidate
# adjacent spans.
	my @tmp = sort(@times);

	my ($DBstart, $DBstop) = split(/\|/,$tmp[$#tmp]);
	debug(100,"DB_EGADS::isOpenSpan: Comparing $DBstop <= $systemtime\n");
	return(1) if ($DBstop  <= $systemtime);
	debug(100,"DB_EGADS::isOpenSpan: Comparing $DBstart <= $maxback\n");
	return(1) if ($DBstart >= $maxback);

	return(0);
}

#
# foundInSpans()
#
sub foundInSpans {
	my $self = shift;
	my $site = shift;
	my $task = shift;
	my ($start,$stop) = @_;

	my $key = $site . "-" . $task;
	my $dayComplete = 0;
	$self->cleanupDBWithKey($key);
	my @tt = $self->{DB}->get_dup($key);
	if ($self->{DB}->get_dup($key) >= 1) {
		my @times = $self->{DB}->get_dup($key);
		foreach my $span (@times) {
			my ($DBstart, $DBstop) = split(/\|/,$span);
			if (($start >= $DBstart) && ($stop <= $DBstop)) {
				return(1);
			}
		}
	}
	return(0);
}

#
# addSpan()
#
# use to add a site/session/day/year/type
# to the EGADS database.
#
# Arguments - site, session code, day, year, data type
# Returns - key into the database if successful (or if the
#           session already exists), undefined on error.
#
sub addSpanWithKey {
	my $self = shift;
	my $key = shift;
	my ($start, $stop) = @_;

	$self->{DB}->put("$key", join("|",$start,$stop));

	$self->{DB}->sync();

	return(0);
}

sub addSpan {
	my $self = shift;
	my $site = shift;
	my $taskid = shift;

	my $key = $site . "-" . $taskid;

	$self->addSpanWithKey($key,@_);
	
	$self->cleanupDBWithKey($key);

}


#
# delSpan()
#
# use to delete a site/session/day/year/type
# from the EGADS database.
#
# Arguments - site, session code, day, year, data type
# Returns - key deleted from database if successful, undefined on error.
#
sub delSpan {
	my $self = shift;
	my $site = shift;
	my $taskid = shift;
	my ($start, $stop) = @_;

#This is tricky

	return $site;
}



sub _open {
	my $self = shift;
	my $dbFileName = $self->{dbFileName};

	$DB_BTREE->{'flags'} = R_DUP;

	$self->{DB} = tie %{$self->{_DB}}, "DB_File", $dbFileName,
					O_RDWR|O_CREAT, 0644, $DB_BTREE;
	unless ($self->{DB}) {
		warn "DB_EGADS - Can't open $dbFileName: $!\n";
		return 0;
	}
	return 1;
}


1;

