package GPSSite;

use Debug qw(debug addToSumFile);
use Data::Dumper;
use strict;
use Exporter;
use URI::URL;

use vars qw(@ISA $VERSION);

@ISA = qw(Exporter);

$VERSION = do {my @r=(q$Revision: 1.1 $=~/\d+/g);sprintf "%d."."%02d"x$#r,@r};

sub new {
	my $class     = shift;
	my $data      = shift;

	# Fill the hash with the remaining arguments to new.
	my $self = {};

	$self->{data} = $data;

	bless $self, $class;
	$self->unlock();
	$self;

}

sub getSiteHash {
	my $self = shift;

	{
		code => $self->{data}->{code},
		rcvr_type => $self->{data}->{receiverinfo}->{type},
		rcvr_model => $self->{data}->{receiverinfo}->{model},
	}
}

sub getRxType {
	my $self = shift;
	return($self->{data}->{receiverinfo}->{type});
}

sub getRxModel {
	my $self = shift;
	return($self->{data}->{receiverinfo}->{model});
}

sub getComm {
	my $self = shift;

	{
		id => '',
		telem => $self->{data}->{comm}->{telemetry},
		telem_spec => $self->{data}->{comm}->{telem_spec},
		area_code => $self->{data}->{comm}->{phone}->{areacode},
		phone_num => $self->{data}->{comm}->{phone}->{number},
		freewave => $self->{data}->{comm}->{freewave}->{slave},
		repeater1 => $self->{data}->{comm}->{freewave}->{repeater1},
		repeater2 => $self->{data}->{comm}->{freewave}->{repeater2},
		connecttype => $self->{data}->{comm}->{type},
		baud => $self->{data}->{comm}->{baud},
	};
}

sub name {
	shift->{data}->{code};
}

sub getSessions {
	my $self = shift;
	$self->makeSessions();
#	return wantarray ?  @{$self->{data}->{sessions}} : $#{$self->{data}->{sessions}} + 1;
}

sub makeSessions {
	my $self = shift;

	my @sessions;
	if ($self->{data}->{sessions}->{type} eq "simple") {
		my $seconds = $self->{data}->{sessions}->{seconds};
		my $sample  = $self->{data}->{sessions}->{sample};
		my $mask    = $self->{data}->{sessions}->{mask};
		my $minsv   = $self->{data}->{sessions}->{minsv};
		
		my $id = 0;
		my $start = 0;
		my $stop = $seconds - 1;
		debug(75,"GPSSite::makeSessions: seconds = $seconds (should be positive)\n");
		while ($start < 86400) {
			$stop = 86399 if ($stop >= 86400);
			last if ($stop < $start);
			my $tmp = {
				id     => $id,
				start  => $start,
				stop   => $stop,
				sample => $sample,
				mask   => $mask,
				minsv  => $minsv,
			};
			push(@sessions, $tmp);
			$id++;
			$start += $seconds;
			$stop = $start + ($seconds - 1);
		}
	}
	return(@sessions);
}

sub getSession {
	my $self = shift;
	my $num = shift;

	my $numsess = $self->getSessions();
	return undef if ($num > $numsess);

	my @return;
	my %tmp;
	for (my $i = 0; $i < $self->getSessions(); $i++) {
		my $key = $self->{data}->{sessions}->[$i]->{start};
		$tmp{$key} = $i;
	}

	my @order = sort(keys(%tmp));
	my $i = $tmp{$order[$num-1]};

	push(@return, $self->{data}->{sessions}->[$i]->{id},
			$self->{data}->{sessions}->[$i]->{start},
			$self->{data}->{sessions}->[$i]->{stop},
			$self->{data}->{sessions}->[$i]->{sample},
			$self->{data}->{sessions}->[$i]->{mask},
			$self->{data}->{sessions}->[$i]->{minsv});
	@return
}

#Given
# client name
# time to keep lock
sub lock {
	my $self = shift;
	$self->setlock(1,@_);
}

sub unlock {
	my $self = shift;
	$self->setlock(0,"");
	
}

sub setlock {
	my $self = shift;
	my $locked = shift;
	my $client = shift;
	my $locktime = (@_) ? shift: -1;

	$locktime += time() if ($locktime >= 0);

	$self->{lock} = {
		islocked => $locked,
		client => $client,
		time => $locktime
	};

	debug(25, "GPSSite::setlock: ");
	if ($locked) {
		debug(25, "Locking $self->{data}->{code} for $client");
		debug(25, " until " . gmtime($locktime)) if ($locktime >= 0);
	} else {
		debug(25, "UNLocking $self->{data}->{code}");
	}
	debug(25, "\n");

	return($self->isLocked());

}

sub isLocked {
	my $self = shift;

	return($self->{lock}->{islocked}) unless ($self->{lock}->{islocked});

	my $tmp = time();
	debug(25, "GPSSite::isLocked: Checking lock time for $self->{data}->{code} -- ");
	if ($self->{lock}->{time} < $tmp) {
		debug(25, " Lock TIMED OUT\n");
		$self->unlock();
	} else {
		debug(25," Locked for " . ($self->{lock}->{time} - $tmp) . " seconds by $self->{lock}->{client}\n");
	}

	return($self->{lock}->{islocked});
}

sub isLockedBy {
	my $self = shift;
	my $client = (@_) ? shift: "";

	return(0) if ($client eq "");
	return(0) unless $self->isLocked();

	return(1) if ($self->{lock}->{client} eq $client);

}

sub getLockClient {
	my $self = shift;

	return("") unless $self->isLocked();

	return($self->{lock}->{client});
}

sub isAllowedTime {
	my $self = shift;

	my $daysec = time() % 86400;

	print Dumper($self->{data}->{allowedtimes}) . "\n";

	for (my $i = 0; $i <= $#{$self->{data}->{allowedtimes}}; $i++) {
		my $start = $self->{data}->{allowedtimes}->[$i]->{start};
		my $stop = $self->{data}->{allowedtimes}->[$i]->{stop};
		return(1) if (($daysec > $start) && ($daysec < $stop));
	}

	return(0);
}


1;

