#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use DateTime::Precise;
use POSIX qw(strftime setuid setgid getuid getgid geteuid getegid);
use Digest::MD5;
use File::Spec::Functions;
use File::stat;

use English;
use Getopt::Long;

### DEFAULT OPTIONS
my $opt_soap = 1;
my $opt_daemon = 0;
my $progtitle = "EGADS_Client";

my $configs = {
	user => "egads",
	server => "localhost",
	serverport => "43012",
	filer => "",
	filerport => "",
	datadir => "/home/egads/EGADS/data/raw/",
	fatdir => "/home/egads/EGADS/data/fats/",
	sumdir => "/home/egads/EGADS/data/sums/",
	sharc => "/usr/bin/sharc",
	debug => "100",
	hubname => ""
};
###

$0 = $progtitle . " " . join(" ",@ARGV);

$| = 1;

our (	$opt_server,
	$opt_filer,
	$opt_datadir,
	$opt_fatdir,
	$opt_sumdir,
	$opt_debug,
	$opt_sharc,
	$opt_user,
	$opt_hubname,
	$opt_configfile
);

my @options = qw(
	user=s
	configfile=s
	server=s
	filer=s
	datadir=s
	atdir=s
	sumdir=s
	debug=i
	sharc=s
	hubname=s
);

my $return = GetOptions('soap!' => \$opt_soap, 'daemon!' => \$opt_daemon, @options);

print "opt_soap = $opt_soap\n";

die "UNSuccessful argument parsing\n" unless ($return);

if ($opt_configfile) {
	die "Cannot find $opt_configfile\n" unless (-f $opt_configfile);
	ReadConfig($opt_configfile,$configs);
}

#Set Debug Level
my $DEBUG = ($opt_debug) ? $opt_debug : $configs->{debug};

#Set hub name
my $hubname = ($opt_hubname) ? $opt_hubname : $configs->{hubname};
die "No hubname specified\n" if ($hubname eq "");

#Set User
my $user = ($opt_user) ? $opt_user : $configs->{user};
die "Do not run as root" if ($user eq "root");
my $uid = getpwnam($user);
my $gid = getgrnam($user);
die "Cannot get uid for $user" unless ($uid);
die "Cannot get gid for $user" unless ($gid);
die "Cannot setuid to 0" if ($uid eq 0);
die "Cannot setgid to 0" if ($gid eq 0);
setuid($uid);
setgid($gid);
$EUID = $uid;
$EGID = $gid;
die "Set UID failed" if (getuid() != $uid);
die "Set GID failed" if (getgid() != $gid);
die "Set EUID failed" if (geteuid() != $uid);
die "Set EGID failed" if (getegid() != $gid);
die "Cannot run as group root" if (getgid() == 0);
die "Cannot run as user root" if (getuid() == 0);
die "Cannot run as group root" if (getegid() == 0);
die "Cannot run as user root" if (geteuid() == 0);


#Verify Sharc
my $sharc = ($opt_sharc) ? $opt_sharc : $configs->{sharc};
die "Cannot find sharc executable: $sharc" unless (-x $sharc);


#Verify Data Dir, must be writable
my $datadir = ($opt_datadir) ? $opt_datadir : $configs->{datadir};
$datadir .= "/" unless ($datadir =~ /\/$/);
die "Problem opening $datadir" unless (-d $datadir && -r _ && -w _);

#Verify Fat Dir, set to DataDir if not exist
my $fatdir = ($opt_fatdir) ? $opt_fatdir : $configs->{fatdir};
$fatdir .= "/" unless ($fatdir =~ /\/$/);
unless (-d $fatdir && -r _ && -w _) {
	print "Problem with $fatdir, setting to $datadir\n";
	$fatdir = $datadir;
}

#Verify SUM Dir, set to DataDir if not exist
my $sumdir = ($opt_sumdir) ? $opt_sumdir : $configs->{sumdir};
$sumdir .= "/" unless ($sumdir =~ /\/$/);
unless (-d $sumdir && -r _ && -w _) {
	print "Problem with $sumdir, setting to $datadir\n";
	$sumdir = $datadir;
}

#Get Server
my ($server, $serverport);
$_ = ($opt_server) ? $opt_server : $configs->{server};
if (/(\S+):(\d+)/) {
	$server = $1;
	$serverport = $2;
} else {
	$server = $_;
	$serverport = $configs->{serverport};
}

#Get Filer location
my ($filer, $filerport);
$_ = ($opt_filer) ? $opt_filer : $configs->{filer};
if (/(\S+):(\d+)/) {
	$filer = $1;
	$filerport = $2;
} else {
	$filer = $_;
	$filerport = $configs->{filerport};
}
$filer = $server if ($filer eq "");
$filerport = $serverport+1 if ($filerport eq "");

if ( $opt_soap ) {
	require "SOAP/Lite.pm";
	import SOAP::Lite +autodispatch =>
			uri => 'http://egadsserver/Server',
			proxy => 'http://' . $server . ":" . $serverport,
			on_fault => sub { my ($soap, $res) = @_;
				die ref $res ? $res->faultstring : $soap->transport->status, "\n";
			};
} else {
	require "Server.pm";
	require "Filer.pm";
}


my $processid = $$;
my $process_time = time();

print "Connecting to $server using $hubname\n";
my $Server = Server->new($hubname);
print "Calling releaseLocks\n";
$Server->releaseLocks();
undef $Server;

do {
	if ($opt_soap) {
		import SOAP::Lite +autodispatch =>
			uri => 'http://egadsserver/Server',
			proxy => 'http://' . $server . ":" . $serverport,
			on_fault => sub { my ($soap, $res) = @_;
				die ref $res ? $res->faultstring : $soap->transport->status, "\n";
			};
	}
	$Server = Server->new($hubname);
	while ((my $sleeptime = $Server->setupJob()) != 0) {
		print "Sleeping for $sleeptime\n";
		sleep($sleeptime);
	}
		
	my ($code, $port, $site, $comm, $file, $tasks, $sessions) = $Server->getAllJobInfo();

	my $down = $tasks->{download};
	my $dele = $tasks->{delete};
	my $conf = $tasks->{configure};

	#my $fatfile = $fatdir . $code . "-" . $processid . ".fat";
	#my $sumfile = $sumdir . $code . "-" . $processid . ".sum";
	my $fatfile = $fatdir . $code . ".fat";
	my $sumfile = $sumdir . $code . ".sum";
	unlink $fatfile if (-f $fatfile);
	unlink $sumfile if (-f $sumfile);
	my $command_line  = $sharc;
	$command_line .= " --debug $DEBUG";
	$command_line .= " --debugfile -";
	$command_line .= " --port \"$port->{device}\"";
	$comm->{baud} = $port->{baud} if ($comm->{baud} eq "");
	$command_line .= " --baud \"$comm->{baud}\"";
	$command_line .= " --filetemplate \"$file->{filetemplate}\"";
	$command_line .= " --datadir \"$datadir\"";
	$command_line .= " --fatfile \"$fatfile\"";
	$command_line .= " --sumfile \"$sumfile\"";
	foreach my $session (@{$sessions}) {
		my $start = $session->{start};
		my $stop = $session->{stop};
		my $mask = $session->{mask};
		my $minsv = $session->{minsv};
		my $sample = $session->{sample};
		$command_line .= " --session \"$start,$stop,$sample,$mask,$minsv\"";
	}
	$command_line .= " --filetimefrom \"request start\"";

SWITCH: for ($comm->{connecttype}) {
		/freewave/i	&& do {
			$command_line .= " --connectvia freewave --phonenumber \"$comm->{freewave}\"";
			$command_line .= " --Cfr $comm->{repeater1}" if ($comm->{repeater1} ne "");
			$command_line .= ",$comm->{repeater2}" if ($comm->{repeater2} ne "");

			last SWITCH;
		};

		/pots/i		&& do {
			$command_line .= " --connectvia modem  --phonenumber \"$port->{prefix}$comm->{area_code}$comm->{phone_num}\"";
			last SWITCH;
		};
	}

	foreach my $downRef (@{$down}) {
		
		my $spanstart = strftime "%Y/0/%j:%T-", gmtime($downRef->{start_time});
		my $spanstop  = strftime "%Y/0/%j:%T", gmtime($downRef->{stop_time});
		my $session  = $downRef->{session};
		my $span = $spanstart . $spanstop;
		
		$command_line .= " --download \"$span $session\"";
	}
	
	foreach my $deleteRef (@{$dele}) {
		my $spanstart = strftime "%Y/0/%j:%T-", gmtime($deleteRef->{start_time});
		my $spanstop  = strftime "%Y/0/%j:%T", gmtime($deleteRef->{stop_time});
		my $span = $spanstart . $spanstop;
		$command_line .= " --purge \"$span\"";
	}

	my $output = "";
	print "Running $command_line\n";

	open(SHARC,"$command_line |");
	my $i = 0;
	my ($tmp);
	while(read(SHARC,$tmp,1)) {
		print $tmp;
#		$i++;
#		if ($i > 100) {
#			$Server->status($output);
#			$i = 0;
#		}
	}

	close(SHARC);
	my $elaps = time() - $process_time;
	print "elapsed = $elaps\n";

	my @downresults;
	my @delresults;
	if (open(SUM,"< $sumfile")) {
		while (<SUM>) {
			print;
			push(@downresults, $1) if (/DOWNLOAD RESULT:(.*)/);
			push(@delresults, $1) if (/DELETE RESULT:(.*)/);
		}
	}
	close(SUM);
	my $download = [];
	for (@downresults) {
		my @line = split;
		my $range = shift(@line);
		my $status = pop(@line);
		my @filenames = @line;
		my ($s, $e);
		if ($range =~ /(\d+)\/(\d+)-(\d+)\/(\d+)/) {
			my ($sw, $ss, $ew, $es) = ($1, $2, $3, $4);
			my $tmp = DateTime::Precise->new();
			$tmp->set_from_gps_week_seconds($sw, $ss);
			$s = $tmp->unix_seconds_since_epoch();
			$tmp = DateTime::Precise->new();
			$tmp->set_from_gps_week_seconds($ew, $es);
			$e = $tmp->unix_seconds_since_epoch();
		}
		if ($status eq "SUCCESS") {
			$tmp = {
				start_time => $s,
				stop_time => $e,
				status => "success",
				numfiles => -1,
				files => [ ]
			};
			foreach my $file (@filenames) {
				print "Checking file: $file\n";
				my $ff = catfile($datadir, $file);
				my $sb = stat($ff);
				if (! $sb ) {
					print "Problem getting stat of $ff\n";
					next;
				}
				next unless (open(MD5,"<$ff"));
				my $ctx = Digest::MD5->new;
				$ctx->addfile(*MD5);
				if ($sb->size > 0) {
					my $filetmp = {
						filename => $file,
						size => $sb->size,
						md5sum => $ctx->hexdigest
					};
					push(@{$tmp->{files}}, $filetmp);
					$tmp->{numfiles} = $#{$tmp->{files}};
				}
			}
			$return = filetransfer($code,$tmp);
			push(@{$download}, $tmp) if ($return == 0);
		}
	}

	my $delete = [];
	for (@delresults) {
		my ($range, $status) = split;
		my ($s, $e);
		if ($range =~ /(\d+)\/(\d+)-(\d+)\/(\d+)/) {
			my ($sw, $ss, $ew, $es) = ($1, $2, $3, $4);
			my $tmp = DateTime::Precise->new();
			$tmp->set_from_gps_week_seconds($sw, $ss);
			$s = $tmp->unix_seconds_since_epoch();
			$tmp = DateTime::Precise->new();
			$tmp->set_from_gps_week_seconds($ew, $es);
			$e = $tmp->unix_seconds_since_epoch();
		}
		if ($status eq "SUCCESS") {
			my $tmp = {
				start_time => $s,
				stop_time => $e,
				status => "success",
			};
			push(@{$delete}, $tmp);
		}
	}

	my $conf_status = {
			start_time => 0,
			stop_time => time()
	};

	my $retval = { download => $download,
			delete => $delete,
			configure => [ $conf_status ],
			};
#	$Server->releaseJob($download, $delete, $conf_status, $receiver_config)
	$Server->releaseJob($retval);

	
} while ($opt_daemon);

sleep 2;

exit(); 

sub filetransfer {
	my $site = shift;
	my $info = shift;

	return(0) if ($info->{numfiles} < 0);
	my $return = 1;
	my $start = $info->{start_time};
	my $stop = $info->{stop_time};
	if ( $opt_soap ) {
		print "Setting Filer to $filer:$filerport\n";
		import SOAP::Lite +autodispatch =>
			uri => 'http://egadsserver/Filer',
			proxy => 'http://' . $filer . ":" . $filerport,
			on_fault => sub { my ($soap, $res) = @_;
				die ref $res ? $res->faultstring : $soap->transport->status, "\n";
			};
	
	}
	foreach my $file (@{$info->{files}}) {
		my $info = {
			filename => $file->{filename},
			size => $file->{size},
			md5sum => $file->{md5sum},
			data => ""
		};

		my $ff = catfile($datadir, $info->{filename});
		open(FH,"< $ff") or return(1);
		binmode(FH);
		while (<FH>) { $info->{data} .= $_; }
		return(1) unless ($info->{size} == length($info->{data}));
		print "Calling Filer $site, $start, $stop, $info->{filename}\n";
		$return = Filer->new($site, $start, $stop, $info);

	}

	if ($opt_soap) {
		print "Restoring server setting to $server:$serverport\n";
		import SOAP::Lite +autodispatch =>
			uri => 'http://egadsserver/Server',
			proxy => 'http://' . $server . ":" . $serverport,
			on_fault => sub { my ($soap, $res) = @_;
				die ref $res ? $res->faultstring : $soap->transport->status, "\n";
			};
	}
	return($return);
	
}


sub ReadConfig {
	my $configfile = shift;
	my $configs = shift;
	open(HI, "<$configfile") || die "cannot open config file: $configfile\n";

	while (<HI>) {
		my ($variable, $value) = split;
		$configs->{$variable} = $value if (exists($configs->{$variable}));
	}
	return();
}


__END__
