#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use DateTime::Precise;
use POSIX qw(strftime setuid setgid getuid getgid geteuid getegid);
use Digest::MD5;
use File::Spec::Functions;
use File::stat;
use File::Copy;

use English;
use Getopt::Long;

### DEFAULT OPTIONS
my $opt_soap = 1;
my $opt_daemon = 0;
my $progtitle = "EGADS_Client";

my $configs = {
	user => "egads2",
	filer => "jbay.gps.caltech.edu",
	filerport => "43013",
	debug => "100",
};
###

$0 = $progtitle . " " . join(" ",@ARGV);

$| = 1;

our (	$opt_filer,
	$opt_debug,
	$opt_user,
	$opt_configfile,
	$opt_file
);

my @options = qw(
	user=s
	configfile=s
	filer=s
	debug=i
	file=s
);

my $return = GetOptions('soap!' => \$opt_soap, 'daemon!' => \$opt_daemon, @options);

print "opt_soap = $opt_soap\n";

die "UNSuccessful argument parsing\n" unless ($return);

if ($opt_configfile) {
	die "Cannot find $opt_configfile\n" unless (-f $opt_configfile);
	ReadConfig($opt_configfile,$configs);
}

#Set Debug Level
my $DEBUG = ($opt_debug) ? $opt_debug : $configs->{debug};

#Set User
my $user = ($opt_user) ? $opt_user : $configs->{user};
die "Do not run as root" if ($user eq "root");
my $uid = getpwnam($user);
my $gid = getgrnam($user);
die "Cannot get uid for $user" unless ($uid);
die "Cannot get gid for $user" unless ($gid);
die "Cannot setuid to 0" if ($uid eq 0);
die "Cannot setgid to 0" if ($gid eq 0);
setuid($uid);
setgid($gid);
$EUID = $uid;
$EGID = $gid;
die "Set UID failed" if (getuid() != $uid);
die "Set GID failed" if (getgid() != $gid);
die "Set EUID failed" if (geteuid() != $uid);
die "Set EGID failed" if (getegid() != $gid);
die "Cannot run as group root" if (getgid() == 0);
die "Cannot run as user root" if (getuid() == 0);
die "Cannot run as group root" if (getegid() == 0);
die "Cannot run as user root" if (geteuid() == 0);


#Get Filer location
my ($filer, $filerport);
$_ = ($opt_filer) ? $opt_filer : $configs->{filer};
if (/(\S+):(\d+)/) {
	$filer = $1;
	$filerport = $2;
} else {
	$filer = $_;
	$filerport = $configs->{filerport};
}


die "--file not specified" unless ($opt_file);

my $myfile = $opt_file if (-f $opt_file);
if (-f $opt_file) {
	$myfile = $opt_file;
} else {
	die "file $opt_file doesn't exist";
}

unless ($opt_soap) {
	require "Filer.pm";
}


my $processid = $$;
my $process_time = time();

my $filename = "";
my $tmp_time = DateTime::Precise->new();
my $s;
my $e;
my $code;
if ($myfile =~ /(....)_.{4,5}_\d{4}_\d{6}_(\d\d)_(\d\d\d)_(.)\..{2,3}/) {
	$filename = $myfile;
	$code = $1;
	my $yr = $2;
	my $day = $3;
	$tmp_time->set_time("YDHMS",$yr+2000, $day, 0,0,0);
	$s = $tmp_time->unix_seconds_since_epoch();
	$tmp_time->set_time("YDHMS",$yr+2000, $day, 23,59,59);
	$e = $tmp_time->unix_seconds_since_epoch();

} else {
	if ($myfile =~ /R(....)(.)(0\d)\.(\d\d\d)/) {
		$code = $1;
		my $sess = $2;
		my $yr = $3;
		my $day = $4;
		$tmp_time->set_time("YDHMS",$yr+2000, $day, 0,0,0);
		$s = $tmp_time->unix_seconds_since_epoch();
		$tmp_time->set_time("YDHMS",$yr+2000, $day, 23,59,59);
		$e = $tmp_time->unix_seconds_since_epoch();
	
$|=1;
		my $gpswk = sprintf("%04d", $tmp_time->gps_week());
		my $gpssc = sprintf("%06d", $tmp_time->gps_seconds());
		my $format = "${code}_xxxxx_${gpswk}_${gpssc}_%y_%j_${sess}.z12";
		$filename = $tmp_time->strftime($format);
	
		print "Copying to $filename\n";
		copy($myfile, $filename);
	} elsif ($myfile =~ /(....)_\d{4}_\d{6}_(\d\d)_(\d\d\d)_(.)\..{2,3}/) {
		$code = $1;
		my $yr = $2;
		my $day = $3;
		my $sess = $4;
		$tmp_time->set_time("YDHMS",$yr+2000, $day, 0,0,0);
		$s = $tmp_time->unix_seconds_since_epoch();
		$tmp_time->set_time("YDHMS",$yr+2000, $day, 23,59,59);
		$e = $tmp_time->unix_seconds_since_epoch();
	
		$filename = $tmp_time->strftime("${code}_xxxxx_%G_%g_%y_%j_${sess}.z12");
	
		print "Copying to $filename\n";
		copy($myfile, $filename);
	} else {
		die "bad filename $myfile";
	}
}
my $file = $filename;

my $tmp = {
	start_time => $s,
	stop_time => $e,
	status => "success",
	numfiles => -1,
	files => [ ]
};
print "Checking file: $file\n";
my $ff = $file;
my $sb = stat($ff);
if (! $sb ) {
	print "Problem getting stat of $ff\n";
	exit();
}
next unless (open(MD5,"<$ff"));
my $ctx = Digest::MD5->new;
$ctx->addfile(*MD5);
if ($sb->size > 0) {
	my $filetmp = {
		filename => $file,
		size => $sb->size,
		md5sum => $ctx->hexdigest
	};
	push(@{$tmp->{files}}, $filetmp);
	$tmp->{numfiles} = $#{$tmp->{files}};
}
$return = filetransfer($code,$tmp);

print "return from filetransfer = $return\n";

exit($return); 

sub filetransfer {
	my $site = shift;
	my $info = shift;

	my $return = 1;
	my $start = $info->{start_time};
	my $stop = $info->{stop_time};
	if ( $opt_soap ) {
		require "SOAP/Lite.pm";
		print "Setting Filer to $filer:$filerport\n";
		import SOAP::Lite +autodispatch =>
			uri => 'http://egadsserver/Filer',
			proxy => 'http://' . $filer . ":" . $filerport,
			on_fault => sub { my ($soap, $res) = @_;
				die ref $res ? $res->faultstring : $soap->transport->status, "\n";
			};
	
	}
	foreach my $file (@{$info->{files}}) {
		my $info = {
			filename => $file->{filename},
			size => $file->{size},
			md5sum => $file->{md5sum},
			data => ""
		};

		my $ff = $info->{filename};
		open(FH,"< $ff") or return(1);
		binmode(FH);
		while (<FH>) { $info->{data} .= $_; }
		return(1) unless ($info->{size} == length($info->{data}));
		print "Calling Filer $site, $start, $stop, $info->{filename}\n";
		$return = Filer->new($site, $start, $stop, $info);

	}

	return($return);
	
}


sub ReadConfig {
	my $configfile = shift;
	my $configs = shift;
	open(HI, "<$configfile") || die "cannot open config file: $configfile\n";

	while (<HI>) {
		my ($variable, $value) = split;
		$configs->{$variable} = $value if (exists($configs->{$variable}));
	}
	return();
}


__END__
