#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use DateTime::Precise;
use POSIX qw(strftime setuid setgid getuid getgid geteuid getegid);
use Digest::MD5;
use File::Spec::Functions;
use File::stat;
use Net::SMTP;

use English;
use Getopt::Long;

### DEFAULT OPTIONS
my $opt_soap = 1;
my $opt_daemon = 0;
my $progtitle = "Reporter";

my @default_mail = qw(
	stark@dukester.com
	aspiotes@gps.caltech.edu
	svanwyk@gps.caltech.edu
	hudnut@gps.caltech.edu
	nking@usgs.gov
);

my $configs = {
	server => "jbay.gps.caltech.edu",
	serverport => "43012",
	back => "7",
        mail => "",
	hubname => "reporter"
};
###

our (	$opt_server,
	$opt_hubname,
	$opt_back,
	$opt_mail,
	$opt_configfile
);

my @options = qw(
	configfile=s
	server=s
	back=s
	mail=s
	hubname=s
);

my $return = GetOptions('soap!' => \$opt_soap, 'daemon!' => \$opt_daemon, @options);

die "UNSuccessful argument parsing\n" unless ($return);

if ($opt_configfile) {
	die "Cannot find $opt_configfile\n" unless (-f $opt_configfile);
	ReadConfig($opt_configfile,$configs);
}

#Set hub name
my $hubname = ($opt_hubname) ? $opt_hubname : $configs->{hubname};
die "No hubname specified\n" if ($hubname eq "");

#Set days back
my $back = ($opt_back) ? $opt_back : $configs->{back};
die "No time back specified\n" if ($back eq "");

#Set days back
my $mail = ($opt_mail) ? $opt_mail : $configs->{mail};
my @mail;
if ($mail eq "default") {
	@mail = @default_mail;
} elsif ($mail ne "") {
	@mail = split(/,/,$mail);
}

#Get Server
my ($server, $serverport);
$_ = ($opt_server) ? $opt_server : $configs->{server};
if (/(\S+):(\d+)/) {
	$server = $1;
	$serverport = $2;
} else {
	$server = $_;
	$serverport = $configs->{serverport};
}

if ( $opt_soap ) {
	require "SOAP/Lite.pm";
	import SOAP::Lite +autodispatch =>
			uri => 'http://egadsserver/Server',
			proxy => 'http://' . $server . ":" . $serverport,
			on_fault => sub { my ($soap, $res) = @_;
				die ref $res ? $res->faultstring : $soap->transport->status, "\n";
			};
} else {
	require "Server.pm";
}


my $now = time();

my @sites;
foreach (@ARGV) {
	push(@sites,$_) if (/^....$/);
}

my $then = ($now - ($back*86400));
my $Server = Server->new($hubname);
my $report;
my $locks;
my $tasks;
my $printout;
if ($#sites >= 0) {
	$tasks = $Server->getTaskInfo("download",@sites);
} else {
	$tasks = $Server->getTaskInfo("download");
}

my @inactive;
foreach my $site (keys(%{$tasks})) {
	if ($tasks->{$site} == 0) {
		push(@inactive, $site);
		delete($tasks->{$site});
	}
}

my $freq = {};
foreach my $site (keys(%{$tasks})) {
	my $tmpf = $tasks->{$site}->{frequency};
	$freq->{$tmpf} = [] unless ($freq->{$tmpf});
	push(@{$freq->{$tmpf}},$site);
}

if ($#inactive >= 0) {
	$printout .= "Inactive Sites: " . join(" ",@inactive) . "\n";
}

foreach my $tmpf (sort { $a <=> $b } keys(%{$freq})) {
	next unless ($#{$freq->{$tmpf}} >= 0);
	my $numhours = $tmpf / 3600;
	my $tmpnow = ($now - ($now % ($tmpf)));
	$tmpnow--;
	$report = $Server->getReportInfo($then,$tmpnow,@{$freq->{$tmpf}});
	my @tmp = keys(%{$report});
	next unless($#tmp >= 0);
	$locks = $Server->getLockInfo(@{$freq->{$tmpf}});

	my $output = {};
	foreach my $site (sort(keys(%{$report}))) {
		next unless ($#{$report->{$site}} >= 0);
		$output->{$site} = "";
	}
	@tmp = keys(%{$output});
	next unless($#tmp >= 0);
	$printout .= "Download Frequency: every $numhours Hour(s)\n";
	my $tag = ($tmpnow - $then) / 55;
	for (my $i = $then; $i < $tmpnow; $i += $tag) {
		foreach my $site (keys(%{$output})) {
			my $mark = " ";
			foreach my $span (@{$report->{$site}}) {
				$mark = "-" if (($i >= $span->{start_time}) &&
							($i <= $span->{stop_time}));
			}
			$output->{$site} .= $mark;
		}
	}
	
	$printout .= "     ";
	my $curday = "";
	my $curtime;
	for ($curtime = $then; $curtime < $tmpnow; $curtime += $tag) {
		my @hi = gmtime($curtime);
		if ($hi[7] ne $curday) {
			$curday = $hi[7];
			$printout .= sprintf("%03d ",$curday);
			$curtime += ($tag * 3);
		} else {
			$printout .= " ";
		}
	}
	$printout .= "\n";

	foreach my $site (sort(keys(%{$output}))) {
		$_ = $output->{$site};
		next unless (/-/);
		$printout .= $site;
		$printout .= " $_";
		if ($locks->{$site}) {
			$printout .= " (" . $locks->{$site} . ")" if ($locks->{$site} ne "system");
		}
		$printout .= "\n";
	}
	
}

if ($#mail >= 0) {
	my $header = "";
	$header .= "Subject: Data Report\n";
	$header .= "To: " . join(",",@mail) . "\n";
	$header .= "\n";
	my $smtp = Net::SMTP->new('mail.gps.caltech.edu',
				Hello => 'gps.caltech.edu',
				Timeout => 30,
				Debug   => 0,
				) || die "new smtp connection";
	$smtp->mail("dgga\@gps.caltech.edu") || die "send";
	$smtp->recipient(@mail) || die "recipient line";
	$smtp->data($header . $printout) || die "data";
	$smtp->quit() || die "quitting";
} else {
	print $printout;
}

exit(); 

sub ReadConfig {
	my $configfile = shift;
	my $configs = shift;
	open(HI, "<$configfile") || die "cannot open config file: $configfile\n";

	while (<HI>) {
		my ($variable, $value) = split;
		$configs->{$variable} = $value if (exists($configs->{$variable}));
	}
	return();
}


__END__
